@@ -24,6 +24,8 @@ DO_IFRAMES = 0
 DO_PNG = 0
 SHOW_SCRIPT = 0
 SHOW_CONSOLE = 0
+SHOW_HISTORY = 0
+SHOW_HAR = 0
 
 # servers
 SPLASH_PORT = 8050
