@@ -9,8 +9,8 @@ from cStringIO import StringIO
 import pytest
 import requests
 from PIL import Image
+from splash.utils import truncated
 from splash.tests.utils import NON_EXISTING_RESOLVABLE
-from splash.tests.utils import SplashServer
 
 
 def https_only(func):
@@ -37,7 +37,7 @@ def skip_proxy(func):
 
 class DirectRequestHandler(object):
 
-    render_format = "html"
+    endpoint = "render.html"
 
     def __init__(self, ts):
         self.ts = ts
@@ -46,21 +46,21 @@ class DirectRequestHandler(object):
     def host(self):
         return "localhost:%s" % self.ts.splashserver.portnum
 
-    def request(self, query, render_format=None, headers=None):
-        url, params = self._url_and_params(render_format, query)
+    def request(self, query, endpoint=None, headers=None):
+        url, params = self._url_and_params(endpoint, query)
         return requests.get(url, params=params, headers=headers)
 
-    def post(self, query, render_format=None, payload=None, headers=None):
-        url, params = self._url_and_params(render_format, query)
+    def post(self, query, endpoint=None, payload=None, headers=None):
+        url, params = self._url_and_params(endpoint, query)
         return requests.post(url, params=params, data=payload, headers=headers)
 
-    def _url_and_params(self, render_format, query):
-        render_format = render_format or self.render_format
+    def _url_and_params(self, endpoint, query):
+        endpoint = endpoint or self.endpoint
         if isinstance(query, dict):
-            url = "http://%s/render.%s" % (self.host, render_format)
+            url = "http://%s/%s" % (self.host, endpoint)
             params = query
         else:
-            url = "http://%s/render.%s?%s" % (self.host, render_format, query)
+            url = "http://%s/%s?%s" % (self.host, endpoint, query)
             params = None
         return url, params
 
@@ -69,7 +69,7 @@ class DirectRequestHandler(object):
 @pytest.mark.usefixtures("print_ts_output")
 class BaseRenderTest(unittest.TestCase):
 
-    render_format = "html"
+    endpoint = "render.html"
     request_handler = DirectRequestHandler
     use_gzip = False
 
@@ -85,63 +85,70 @@ class BaseRenderTest(unittest.TestCase):
 
     def _get_handler(self):
         handler = self.request_handler(self.ts)
-        handler.render_format = self.render_format
+        handler.endpoint = self.endpoint
         return handler
 
-    def request(self, query, render_format=None, headers=None, **kwargs):
-        return self._get_handler().request(query, render_format, headers, **kwargs)
+    def request(self, query, endpoint=None, headers=None, **kwargs):
+        return self._get_handler().request(query, endpoint, headers, **kwargs)
 
-    def post(self, query, render_format=None, payload=None, headers=None, **kwargs):
-        return self._get_handler().post(query, render_format, payload, headers, **kwargs)
+    def post(self, query, endpoint=None, payload=None, headers=None, **kwargs):
+        return self._get_handler().post(query, endpoint, payload, headers, **kwargs)
+
+    def assertStatusCode(self, response, code):
+        msg = (response.status_code, truncated(response.content, 1000))
+        self.assertEqual(response.status_code, code, msg)
 
 
-class _RenderTest(BaseRenderTest):
+class Base(object):
+    # a hack to skip running of a base RenderTest
 
-    @unittest.skipIf(NON_EXISTING_RESOLVABLE, "non existing hosts are resolvable")
-    def test_render_error(self):
-        r = self.request({"url": "http://non-existent-host/"})
-        self.assertEqual(r.status_code, 502)
+    class RenderTest(BaseRenderTest):
 
-    def test_timeout(self):
-        r = self.request({"url": self.mockurl("delay?n=10"), "timeout": "0.5"})
-        self.assertEqual(r.status_code, 504)
+        @unittest.skipIf(NON_EXISTING_RESOLVABLE, "non existing hosts are resolvable")
+        def test_render_error(self):
+            r = self.request({"url": "http://non-existent-host/"})
+            self.assertStatusCode(r, 502)
 
-    def test_timeout_out_of_range(self):
-        r = self.request({"url": self.mockurl("delay?n=10"), "timeout": "999"})
-        self.assertEqual(r.status_code, 400)
+        def test_timeout(self):
+            r = self.request({"url": self.mockurl("delay?n=10"), "timeout": "0.5"})
+            self.assertStatusCode(r, 504)
 
-    @skip_proxy
-    def test_missing_url(self):
-        r = self.request({})
-        self.assertEqual(r.status_code, 400)
-        self.assertTrue("url" in r.text)
+        def test_timeout_out_of_range(self):
+            r = self.request({"url": self.mockurl("delay?n=10"), "timeout": "999"})
+            self.assertStatusCode(r, 400)
 
-    def test_jsalert(self):
-        r = self.request({"url": self.mockurl("jsalert"), "timeout": "3"})
-        self.assertEqual(r.status_code, 200)
+        @skip_proxy
+        def test_missing_url(self):
+            r = self.request({})
+            self.assertStatusCode(r, 400)
+            self.assertTrue("url" in r.text)
 
-    def test_jsconfirm(self):
-        r = self.request({"url": self.mockurl("jsconfirm"), "timeout": "3"})
-        self.assertEqual(r.status_code, 200)
+        def test_jsalert(self):
+            r = self.request({"url": self.mockurl("jsalert"), "timeout": "3"})
+            self.assertStatusCode(r, 200)
 
-    def test_iframes(self):
-        r = self.request({"url": self.mockurl("iframes"), "timeout": "3"})
-        self.assertEqual(r.status_code, 200)
+        def test_jsconfirm(self):
+            r = self.request({"url": self.mockurl("jsconfirm"), "timeout": "3"})
+            self.assertStatusCode(r, 200)
 
-    def test_wait(self):
-        r1 = self.request({"url": self.mockurl("jsinterval")})
-        r2 = self.request({"url": self.mockurl("jsinterval")})
-        r3 = self.request({"url": self.mockurl("jsinterval"), "wait": "0.2"})
-        self.assertEqual(r1.status_code, 200)
-        self.assertEqual(r2.status_code, 200)
-        self.assertEqual(r3.status_code, 200)
-        self.assertEqual(r1.content, r2.content)
-        self.assertNotEqual(r1.content, r3.content)
+        def test_iframes(self):
+            r = self.request({"url": self.mockurl("iframes"), "timeout": "3"})
+            self.assertStatusCode(r, 200)
+
+        def test_wait(self):
+            r1 = self.request({"url": self.mockurl("jsinterval")})
+            r2 = self.request({"url": self.mockurl("jsinterval")})
+            r3 = self.request({"url": self.mockurl("jsinterval"), "wait": "0.2"})
+            self.assertStatusCode(r1, 200)
+            self.assertStatusCode(r2, 200)
+            self.assertStatusCode(r3, 200)
+            self.assertEqual(r1.content, r2.content)
+            self.assertNotEqual(r1.content, r3.content)
 
 
-class RenderHtmlTest(_RenderTest):
+class RenderHtmlTest(Base.RenderTest):
 
-    render_format = "html"
+    endpoint = "render.html"
 
     def test_jsrender(self):
         self._test_jsrender(self.mockurl("jsrender"))
@@ -152,10 +159,10 @@ class RenderHtmlTest(_RenderTest):
 
     def _test_jsrender(self, url):
         r = self.request({"url": url})
-        self.assertEqual(r.status_code, 200)
+        self.assertStatusCode(r, 200)
         self.assertEqual(r.headers["content-type"].lower(), "text/html; charset=utf-8")
-        self.assertTrue("Before" not in r.text)
-        self.assertTrue("After" in r.text)
+        self.assertNotIn("Before", r.text)
+        self.assertIn("After", r.text)
 
     def test_baseurl(self):
         # first make sure that script.js is served under the right url
@@ -167,32 +174,32 @@ class RenderHtmlTest(_RenderTest):
             "url": self.mockurl("baseurl"),
             "baseurl": self.mockurl("baseurl/"),
         })
-        self.assertEqual(r.status_code, 200)
-        self.assertTrue("Before" not in r.text)
-        self.assertTrue("After" in r.text)
+        self.assertStatusCode(r, 200)
+        self.assertNotIn("Before", r.text)
+        self.assertIn("After", r.text)
 
     def test_otherdomain(self):
         r = self.request({"url": self.mockurl("iframes")})
-        self.assertEqual(r.status_code, 200)
-        self.assertTrue('SAME_DOMAIN' in r.text)
-        self.assertTrue('OTHER_DOMAIN' in r.text)
+        self.assertStatusCode(r, 200)
+        self.assertIn('SAME_DOMAIN', r.text)
+        self.assertIn('OTHER_DOMAIN', r.text)
 
     def test_allowed_domains(self):
         r = self.request({'url': self.mockurl('iframes'), 'allowed_domains': 'localhost'})
-        self.assertEqual(r.status_code, 200)
-        self.assertTrue('SAME_DOMAIN' in r.text)
-        self.assertFalse('OTHER_DOMAIN' in r.text)
+        self.assertStatusCode(r, 200)
+        self.assertIn('SAME_DOMAIN', r.text)
+        self.assertNotIn('OTHER_DOMAIN', r.text)
 
     def test_viewport(self):
         r = self.request({'url': self.mockurl('jsviewport'), 'viewport': '300x400'})
-        self.assertEqual(r.status_code, 200)
+        self.assertStatusCode(r, 200)
         self.assertIn('300x400', r.text)
 
     def test_nonascii_url(self):
         nonascii_value =  u'тест'.encode('utf8')
         url = self.mockurl('getrequest') + '?param=' + nonascii_value
         r = self.request({'url': url})
-        self.assertEqual(r.status_code, 200)
+        self.assertStatusCode(r, 200)
         self.assertTrue(
             repr(nonascii_value) in r.text or  # direct request
             urllib.quote(nonascii_value) in r.text,  # request in proxy mode
@@ -201,14 +208,14 @@ class RenderHtmlTest(_RenderTest):
 
     def test_result_encoding(self):
         r1 = requests.get(self.mockurl('cp1251'))
-        self.assertEqual(r1.status_code, 200)
+        self.assertStatusCode(r1, 200)
         self.assertEqual(r1.encoding, 'windows-1251')
-        self.assertTrue(u'проверка' in r1.text)
+        self.assertIn(u'проверка', r1.text)
 
         r2 = self.request({'url': self.mockurl('cp1251')})
-        self.assertEqual(r2.status_code, 200)
+        self.assertStatusCode(r2, 200)
         self.assertEqual(r2.encoding, 'utf-8')
-        self.assertTrue(u'проверка' in r2.text)
+        self.assertIn(u'проверка', r2.text)
 
     @skip_proxy
     def test_404_get(self):
@@ -229,13 +236,13 @@ class RenderHtmlTest(_RenderTest):
     def assertResponse200Get(self, code):
         url = self.mockurl('getrequest') + '?code=%d' % code
         r = self.request({'url': url})
-        self.assertEqual(r.status_code, 200)
+        self.assertStatusCode(r, 200)
         self.assertIn("GET request", r.text)
 
 
-class RenderPngTest(_RenderTest):
+class RenderPngTest(Base.RenderTest):
 
-    render_format = "png"
+    endpoint = "render.png"
 
     def test_ok(self):
         self._test_ok(self.mockurl("jsrender"))
@@ -245,7 +252,7 @@ class RenderPngTest(_RenderTest):
         self._test_ok(self.ts.mockserver.https_url("jsrender"))
 
     def _test_ok(self, url):
-        r = self.request("url=%s" % url)
+        r = self.request({"url": url})
         self.assertPng(r, width=1024, height=768)
 
     def test_width(self):
@@ -260,20 +267,25 @@ class RenderPngTest(_RenderTest):
         for arg in ('width', 'height'):
             for val in (-1, 99999):
                 url = self.mockurl("jsrender")
-                r = self.request("url=%s&%s=%d" % (url, arg, val))
-                self.assertEqual(r.status_code, 400)
+                r = self.request({"url": url, arg: val})
+                self.assertStatusCode(r, 400)
 
     def test_viewport_full_wait(self):
         r = self.request({'url': self.mockurl("jsrender"), 'viewport': 'full'})
-        self.assertEqual(r.status_code, 400)
+        self.assertStatusCode(r, 400)
 
         r = self.request({'url': self.mockurl("jsrender"), 'viewport': 'full', 'wait': 0.1})
-        self.assertEqual(r.status_code, 200)
+        self.assertStatusCode(r, 200)
 
-    def test_viewport_checks(self):
-        for viewport in ['99999x1', '1x99999', 'foo', '1xfoo', 'axe', '9000x9000', '-1x300']:
+    def test_viewport_invalid(self):
+        for viewport in ['foo', '1xfoo', 'axe', '-1x300']:
             r = self.request({'url': self.mockurl("jsrender"), 'viewport': viewport})
-            self.assertEqual(r.status_code, 400)
+            self.assertStatusCode(r, 400)
+
+    def test_viewport_out_of_bounds(self):
+        for viewport in ['99999x1', '1x99999', '9000x9000']:
+            r = self.request({'url': self.mockurl("jsrender"), 'viewport': viewport})
+            self.assertStatusCode(r, 400)
 
     def test_viewport_full(self):
         r = self.request({'url': self.mockurl("tall"), 'viewport': 'full', 'wait': 0.1})
@@ -288,7 +300,7 @@ class RenderPngTest(_RenderTest):
         self.assertPixelColor(r, 30, 30, (255,255,255,255))
 
     def assertPng(self, response, width=None, height=None):
-        self.assertEqual(response.status_code, 200)
+        self.assertStatusCode(response, 200)
         self.assertEqual(response.headers["content-type"], "image/png")
         img = Image.open(StringIO(response.content))
         self.assertEqual(img.format, "PNG")
@@ -303,9 +315,9 @@ class RenderPngTest(_RenderTest):
         self.assertEqual(color, img.getpixel((x, y)))
 
 
-class RenderJsonTest(_RenderTest):
+class RenderJsonTest(Base.RenderTest):
 
-    render_format = 'json'
+    endpoint = 'render.json'
 
     def test_jsrender_html(self):
         self.assertSameHtml(self.mockurl("jsrender"))
@@ -426,9 +438,9 @@ class RenderJsonTest(_RenderTest):
         r1 = self.request({"url": self.mockurl("jsinterval"), 'html': 1})
         r2 = self.request({"url": self.mockurl("jsinterval"), 'html': 1})
         r3 = self.request({"url": self.mockurl("jsinterval"), 'wait': 0.2, 'html': 1})
-        self.assertEqual(r1.status_code, 200)
-        self.assertEqual(r2.status_code, 200)
-        self.assertEqual(r3.status_code, 200)
+        self.assertStatusCode(r1, 200)
+        self.assertStatusCode(r2, 200)
+        self.assertStatusCode(r3, 200)
 
         html1 = r1.json()['html']
         html2 = r2.json()['html']
@@ -438,7 +450,7 @@ class RenderJsonTest(_RenderTest):
 
     def test_result_encoding(self):
         r = self.request({'url': self.mockurl('cp1251'), 'html': 1})
-        self.assertEqual(r.status_code, 200)
+        self.assertStatusCode(r, 200)
         html = r.json()['html']
         self.assertTrue(u'проверка' in html)
         self.assertTrue(u'1251' in html)
@@ -473,15 +485,15 @@ class RenderJsonTest(_RenderTest):
     def _do_same_requests(self, url, params, other_format):
         query = {'url': url}
         query.update(params or {})
-        r1 = self.request(query, render_format='json')
-        r2 = self.request(query, render_format=other_format)
-        self.assertEqual(r1.status_code, 200)
-        self.assertEqual(r2.status_code, 200)
+        r1 = self.request(query, endpoint='render.json')
+        r2 = self.request(query, endpoint='render.' + other_format)
+        self.assertStatusCode(r1, 200)
+        self.assertStatusCode(r2, 200)
         return r1, r2
 
 
 class RenderJsonHistoryTest(BaseRenderTest):
-    render_format = 'json'
+    endpoint = 'render.json'
 
     def test_history_simple(self):
         self.assertHistoryUrls(
@@ -541,7 +553,7 @@ class RenderJsonHistoryTest(BaseRenderTest):
 
 
 class IframesRenderTest(BaseRenderTest):
-    render_format = 'json'
+    endpoint = 'render.json'
 
     def test_basic(self):
         self.assertIframesText('IFRAME_1_OK')
